{################################################################ LIBPRT.INC }
{ Purpose...: Parallel printer support functions. Supports Epson FX-80
{ DependsOn.: LIB.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

CONST g_cLibPrtMODE_INIT      = '@';             { initialize printer        }
CONST g_cLibPrtMODE_PICA      = 'P';             { pica (standard) mode      }
CONST g_cLibPrtMODE_ELITE     = 'M';             { elite mode                }
CONST g_cLibPrtMODE_COND      = ^O;              { condensed mode            }
CONST g_cLibPrtMODE_EMPH      = 'E';             { emphasized mode           }
CONST g_cLibPrtMODE_LMARGIN   = 'l';             { set left margin           }

CONST g_bLibPrtPAGE_LINES_A4    = 60;            { A4 lines per page         }
CONST g_bLibPrtPAGE_LENGTH_A4   = 65;            { A4 form length            }
CONST g_bLibPrtPAGE_LINES_B5    = 40;            { B5 lines per page         }
CONST g_bLibPrtPAGE_LENGTH_B5   = 45;            { B5 form length            }
CONST g_bLibPrtPAGE_LINES_CONT  = 65;            { continuous skip perf at   }
CONST g_bLibPrtPAGE_LENGTH_CONT = 70;            { continuous form length    }

CONST m_bPORT_PRT_CONTROL = $1C;                 { centronics control port   }
CONST m_bBIT_PRT_BUSY     = $08;                 { printer busy bit          }
CONST m_bBIT_PRT_STROBE   = $10;                 { printer data strobe bit   }
CONST m_bPORT_PRT_DATA    = $08;                 { centronics data port      }

CONST m_iTIMEOUT = 2500;                         { printer timeout * 1mS     }

VAR m_bLineCnt    : BYTE;                        { counter for lines/page    }
VAR m_bPageLines  : BYTE;                        { linbes per page to print  }
VAR m_bPageLength : BYTE;                        { length of page in lines   }
VAR m_bMsgRow     : BYTE;                        { message row               }
VAR m_bMsgCol     : BYTE;                        { message column            }

{************************************************************ implementation }
{=========================================================== LibPrt_Prompt() }
{ Purpose...: Display prompt message and wait for user response
{ Input.....: i_stMsg..: Message to display
{             i_stRsp..: Valid response characters
{             i_cYes...: Character for YES (positive) answer
{             i_fCursor: TRUE= use cursor positioning, FALSE= use WRITELN
{ Result....: TRUE= User response = i_cYes, FLASE= something else
{============================================================================}
FUNCTION LibPrt_Prompt(i_stMsg: g_tst; i_stRsp: g_tst; i_cYes: CHAR;
                       i_fCursor: BOOLEAN): BOOLEAN;

  VAR fFnResult : BOOLEAN;
  VAR cKbd      : CHAR;

BEGIN
  fFnResult := FALSE;

  {----------------------------------------------------- draw prompt message }
  IF (i_fCursor = TRUE) THEN BEGIN
    GOTOXY(m_bMsgRow, m_bMsgCol);
    CLREOL;
  END ELSE BEGIN
    WRITELN;
  END;

  {----------------------------------------------- wait for answer from user }
  WRITE(i_stMsg);
  cKbd := Lib_GetKey(i_stRsp, TRUE);
  fFnResult := (UPCASE(cKbd) = UPCASE(i_cYes));

  {----------------------------- draw continue message if answer is positive }
  IF (i_fCursor = TRUE) THEN BEGIN
    GOTOXY(m_bMsgRow, m_bMsgCol);
    CLREOL;

    IF (fFnResult = TRUE) THEN BEGIN
      WRITE('Continue printing... ');
    END;
  END ELSE BEGIN
    WRITELN;
  END;

  LibPrt_Prompt := fFnResult;
END;
{======================================================= eoF LibPrt_Prompt() }

{============================================================== LibPrt_Out() }
{ Purpose...: Output one character to printer
{============================================================================}
PROCEDURE LibPrt_Out(i_cChar: CHAR);

  VAR bCtrl : BYTE;

BEGIN
  PORT[m_bPORT_PRT_DATA] := ORD(i_cChar);
  bCtrl := PORT[m_bPORT_PRT_CONTROL];
  bCtrl := bCtrl AND (NOT m_bBIT_PRT_STROBE);
  PORT[m_bPORT_PRT_CONTROL] := bCtrl;
  bCtrl := bCtrl OR m_bBIT_PRT_STROBE;
  PORT[m_bPORT_PRT_CONTROL] := bCtrl;
END;
{========================================================== eoP LibPrt_Out() }

{============================================================ LibPrt_Ready() }
{ Purpose...: Check if parallel printer is ready
{ Result....: TRUE= printer is ready, FALSE= it is not
{============================================================================}
FUNCTION LibPrt_Ready: BOOLEAN;

BEGIN
  LibPrt_Ready := (PORT[m_bPORT_PRT_CONTROL] AND m_bBIT_PRT_BUSY) <> 0;
END;
{======================================================== eoF LibPrt_Ready() }

{======================================================== LibPrt_WaitReady() }
{ Purpose...: Wait for about 2.5 seconds for printer to become ready
{ Result....: TRUE= printer is ready, FALSE= it is not
{============================================================================}
FUNCTION LibPrt_WaitReady: BOOLEAN;

  VAR iTimeout : INTEGER;

BEGIN
  iTimeout := m_iTIMEOUT;

  WHILE ((LibPrt_Ready = FALSE) AND (iTimeout <> 0)) DO BEGIN
    DELAY(1);
    iTimeout := iTimeout - 1;
  END;

  LibPrt_WaitReady := (iTimeout <> 0);
END;
{==================================================== eoF LibPrt_WaitReady() }

{============================================================= LibPrt_Char() }
{ Purpose...: Print one character after waiting for printer ready
{ Result....: TRUE= success, FALSE= printer not ready
{============================================================================}
FUNCTION LibPrt_Char(i_cChar: CHAR): BOOLEAN;

  VAR fFnResult : BOOLEAN;

BEGIN
  fFnResult := FALSE;

  IF (LibPrt_WaitReady = TRUE) THEN BEGIN
    LibPrt_Out(i_cChar);
    fFnResult := TRUE;
  END;

  LibPrt_Char := fFnResult;
END;
{========================================================= eoF LibPrt_Char() }

{============================================================ LibPrt_Print() }
{ Purpose...: Print character or string to parallel printer. If printer
{             not ready, display message and ask for retry or abort.
{             Line counter can be reset without printing when calling
{             with i_cPrt = CHR(0) and i_stPrt = ''
{ Input.....: i_cPrt....: Character to print. No print if CHR(0)
{             i_pstPrt..: String to print. If NIL, i_cPrt is printed.
{ Result....: TRUE= success, FALSE= printer not ready and abort selected
{============================================================================}
FUNCTION LibPrt_Print(i_cPrt: CHAR; i_stPrt: g_tst): BOOLEAN;

  VAR fPosCursor : BOOLEAN;
  VAR fPrint     : BOOLEAN;
  VAR bCrLf      : BYTE;
  VAR cKbd       : CHAR;
  VAR bNdx       : BYTE;
  VAR bLen       : BYTE;
  VAR iCnt       : INTEGER;

BEGIN
  fPosCursor := ((m_bMsgRow <> 0) AND (m_bMsgCol <> 0));
  fPrint := TRUE;
  bCrLf := 0;
  bNdx := 1;
  bLen := LENGTH(i_stPrt);

  {----------------------------------------------- check string or character }
  IF ((bLen = 0) AND (i_cPrt <> CHR(0))) THEN BEGIN
    i_stPrt := i_stPrt + i_cPrt;
    bLen := 1;
  END;

  {-------------------------------------- either string or character defined }
  WHILE ((fPrint = TRUE) AND (bNdx <= bLen)) DO BEGIN

    {----------------------------------------------- print line or character }
    IF (LibPrt_Char(i_stPrt[bNdx]) = TRUE) THEN BEGIN

      {---------------------- set linecounter if both CR and LF were printed }
      IF (i_stPrt[bNdx] = g_cCR) THEN BEGIN
        bCrLf := (bCrLf OR 1);
      END;

      IF (i_stPrt[bNdx] = g_cLF) THEN BEGIN
        bCrLf := (bCrLf OR 2);
      END;

      IF (bCrLf = 3) THEN BEGIN
        m_bLineCnt := m_bLineCnt + 1;
        bCrLf := 0;
      END;

      bNdx := bNdx + 1;
    END

    {------------------------------------ printer not ready, display message }
    ELSE BEGIN
      fPrint := LibPrt_Prompt('Printer not ready. <R>etry or <A>bort ',
                              'RrAa', 'R', fPosCursor);
    END;
  END;

  {---------------------------------- issue form feed if page length reached }
  IF ((fPrint = TRUE) AND (m_bPageLines <> 0)) THEN BEGIN

    IF (m_bLineCnt >= m_bPageLines) THEN BEGIN

      WHILE ((m_bLineCnt<(m_bPageLength AND $7F))AND(fPrint=TRUE)) DO BEGIN
        fPrint := LibPrt_Char(g_cCR);

        IF (fPrint = TRUE) THEN BEGIN
          fPrint := LibPrt_Char(g_cLF);
          m_bLineCnt := m_bLineCnt + 1;
        END;
      END;

      m_bLineCnt := 0;

      {---------------------------------------- display paper-change message }
      IF ((m_bPageLength AND $80) = 0) THEN BEGIN
        fPrint := LibPrt_Prompt('Insert paper. <C>ontinue or <A>bort ',
                                'CcAa', 'C', fPosCursor);
      END;
    END;
  END;

  LibPrt_Print := fPrint;
END;
{======================================================== eoF LibPrt_Print() }

{========================================================== LibPrt_SetForm() }
{ Purpose...: Set paper format
{ Input.....: i_bPageLines.: Number of lines per page. 0 disables form feeds
{             i_bPageLength: Physical length of page, in lines
{============================================================================}
PROCEDURE LibPrt_SetForm(i_bPageLength: BYTE; i_bPageLines: BYTE);

BEGIN
  m_bPageLength := i_bPageLength;
  m_bPageLines := i_bPageLines;
END;
{====================================================== eoF LibPrt_SetForm() }

{============================================================ LibPrt_Setup() }
{ Purpose...: Set prompts positioning and optionally reset line counter
{ Input.....: i_bMsgRow....: Screen row to print timeout message, 0=WRITELN
{             i_bMsgCol....: Screen column
{             i_fClrLineCnt: TRUE= clear line counter, FALSE= leave it
{============================================================================}
PROCEDURE LibPrt_Setup(i_bMsgRow: BYTE; i_bMsgCol: BYTE;
                       i_fClrLineCnt: BOOLEAN);

BEGIN
  m_bMsgRow := i_bMsgRow;
  m_bMsgCol := i_bMsgCol;

  IF (i_fClrLineCnt = TRUE) THEN BEGIN
    m_bLineCnt := 0;
  END;
END;
{======================================================== eoF LibPrt_Setup() }

{============================================================= LibPrt_Init() }
{ Purpose...: Initialize module:
{============================================================================}
PROCEDURE LibPrt_Init;

BEGIN
  m_bLineCnt := 0;
  m_bPageLines := g_bLibPrtPAGE_LINES_A4;
  m_bPageLength := g_bLibPrtPAGE_LENGTH_A4;
  m_bMsgRow := 0;
  m_bMsgCol := 0;
END;
{========================================================= eoF LibPrt_Init() }

{############################################################ EOF LIBPRT.INC }