{############################################################## TM990ASM.INC }
{ Purpose...: TM990 include file: Send/receive ASM files
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{*************************************************************** definitions }
CONST m_iASM_BAUD_RATE:  INTEGER  = 300;         { baudrate for .ASM files   }
CONST m_iASM_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iASM_TXDLY_LINE: INTEGER  = 5000;        { tx delay detween lines    }

{************************************************************ implementation }
{============================================================= Asm_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Asm_InitSio(i_fInit: BOOLEAN);

BEGIN

  LibSio_Init(i_fInit, m_iASM_BAUD_RATE, 'E', 7, 1,
              m_iASM_TXDLY_CHAR, m_iASM_TXDLY_LINE);
END;
{========================================================= eoF Asm_InitSio() }

{=========================================================== Asm_WaitChars() }
{ Purpose...: Wait until no more characters ready from SIO, print the
{             received chars
{ Input.....: i_iWait.....: Wait time, -1 = default
{             i_pstPrinter: Address of printer string
{             i_pcLastChar: Where to store the last received character
{             i_pcWaitChar: Return when this char is received. If hit,
{                           i_pcWaitChar^ holds $FF to indicate trigger.
{             i_fWrite....: TRUE= write received chars to screen
{                           FALSE= do not
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_WaitChars(i_iWait: INTEGER; i_pstPrinter: g_tpst;
                       i_pcLastChar: g_tpChar; i_pcWaitChar: g_tpChar;
                       i_fWrite: BOOLEAN): BYTE;

  VAR bSioResult: BYTE;
  VAR bRx       : BYTE;
  VAR pbRx      : ^BYTE;
  VAR iWait     : INTEGER;
  VAR fDone     : BOOLEAN;

BEGIN
  bRx := 0;
  pbRx := PTR(ADDR(bRx));
  bSioResult := g_bLibSioOK;
  iWait := i_iWait;
  fDone := FALSE;

  IF (i_iWait = -1) THEN iWait := 100;

  WHILE ((fDone = FALSE) AND (bSioResult = g_bLibSioOK)) DO BEGIN
    bSioResult := LibSio_RxByte(pbRx, iWait, TRUE);

    IF (bSioResult = g_bLibSioOK) THEN BEGIN
      bRx := (bRx AND $7F);

      IF (i_fWrite = TRUE) THEN BEGIN
        WRITE(CHR(bRx));
      END;

      IF (i_pcWaitChar <> NIL) THEN BEGIN
        IF (CHR(bRx) = i_pcWaitChar^) THEN BEGIN
          fDone := TRUE;
          i_pcWaitChar^ := CHR($FF);
        END;
      END;

      IF (i_pcLastChar <> NIL) THEN BEGIN
        i_pcLastChar^ := CHR(bRx);
      END;

      IF (i_pstPrinter <> NIL) THEN BEGIN
        i_pstPrinter^ := i_pstPrinter^ + CHR(bRx);
      END;
    END;
  END;

  Asm_WaitChars := bSioResult;
END;
{======================================================= eoF Asm_WaitChars() }

{============================================================== Asm_WakeUp() }
{ Purpose...: Try to wake up TM990
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_WakeUp: BYTE;

  VAR bSioResult : BYTE;
  VAR cWaitChar  : CHAR;
  VAR pcWaitChar : ^CHAR;

BEGIN
  bSioResult := g_bLibSioOK;
  pcWaitChar := PTR(ADDR(cWaitChar));
  cWaitChar := '?';

  WRITE('Waking up TM990 ... ');

  WHILE ((bSioResult <> g_bLibSioCANCEL) AND (cWaitChar <> CHR($FF))) DO BEGIN
    bSioResult := LibSio_TxByte(ORD(g_cCR));
    bSioResult := Asm_WaitChars(300, NIL, NIL, pcWaitChar, FALSE);

    IF (cWaitChar = CHR($FF)) THEN BEGIN
      WRITELN('Ready');
    END;
  END;

  Asm_WakeUp := bSioResult;
END;
{========================================================== eoF Asm_WakeUp() }

{=========================================================== Asm_PutBuffer() }
{ Purpose...: Write string to file buffer
{ Input.....: i_stString: String to write
{ Result....: TRUE= ok, FALSE= buffer full
{============================================================================}
FUNCTION Asm_PutBuffer(i_stString: g_tst): BOOLEAN;

  VAR bCnt      : BYTE;
  VAR bLen      : BYTE;
  VAR fFnResult : BOOLEAN;

BEGIN
  bCnt := 0;
  bLen := LENGTH(i_stString);
  fFnResult := TRUE;

  WHILE ((fFnResult = TRUE) AND (bCnt < bLen)) DO BEGIN
    bCnt := bCnt + 1;
    fFnResult := File_PutBuffer(i_stString[bCnt]);
  END;

  Asm_PutBuffer := fFnResult;
END;
{======================================================= eoF Asm_PutBuffer() }

{====================================================== Asm_ReceiveMemFile() }
{ Purpose...: Receive MEM file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_ReceiveMemFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  TYPE tstAddr = STRING[10];

  VAR bSioResult  : BYTE;
  VAR cKbd        : CHAR;
  VAR cRx         : CHAR;
  VAR bRx         : BYTE;
  VAR bCnt        : BYTE;
  VAR pbRx        : ^BYTE;
  VAR iTimeout    : INTEGER;
  VAR iFromAddr   : INTEGER;
  VAR iToAddr     : INTEGER;
  VAR iExecAddr   : INTEGER;
  VAR piAddr      : ^INTEGER;
  VAR stAddr      : tstAddr;
  VAR cWait       : CHAR;
  VAR pcWait      : ^CHAR;
  VAR fDone       : BOOLEAN;
  VAR fCancel     : BOOLEAN;
  VAR fRx         : BOOLEAN;
  VAR fTerm       : BOOLEAN;
  VAR stComment   : g_tst;

BEGIN
  iTimeout := -1;
  pbRx := PTR(ADDR(cRx));
  pcWait := PTR(ADDR(cWait));
  bSioResult := g_bLibSioOK;
  fCancel := TRUE;
  fDone := FALSE;
  fRx := FALSE;
  fTerm := FALSE;

  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');

  {--------------------------------- get from and to addresses from keyboard }
  piAddr := PTR(ADDR(iFromAddr));
  WRITE('From:');

  IF (Lib_GetNum(piAddr, NIL, 4, TRUE)) THEN BEGIN
    WRITE('  To:');
    piAddr := PTR(ADDR(iToAddr));

    IF (Lib_GetNum(piAddr, NIL, 4, TRUE)) THEN BEGIN
      fCancel := FALSE;
      WRITE('  Exec (0000=none):');
      piAddr := PTR(ADDR(iExecAddr));

      IF (Lib_GetNum(piAddr, NIL, 4, TRUE)) THEN BEGIN
        WRITELN;
        WRITE('Comment: ');
        READ(stComment);
        WRITELN;
      END;
    END;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (fCancel = FALSE) THEN BEGIN
    Asm_InitSio(TRUE);
    LibSio_RxClear;

    {------------------------------------ make sure TM990 is in command mode }
    bSioResult := Asm_WakeUp;
    fCancel := (bSioResult = g_bLibSioCANCEL);
  END;

  {-------------------------------------------- write comment to file buffer }
  IF ((fCancel = FALSE) AND (LENGTH(stComment) <> 0)) THEN BEGIN
    stComment := ';' + g_stCRLF + '; ' + stComment;
    stComment := stComment + g_stCRLF + ';' + g_stCRLF;
    fCancel := (Asm_PutBuffer(stComment) = FALSE);
  END;

  {---------------------- write 'M' command and start address to file buffer }
  IF (fCancel = FALSE) THEN BEGIN
    stAddr := g_stCRLF + 'M' + Lib_CnvBinToHex(iFromAddr, 4) + g_cCR;
    fCancel := (Asm_PutBuffer(stAddr) = FALSE);
  END;

  {-------------------------------------- write command and address to TM990 }
  IF (fCancel = FALSE) THEN BEGIN
    bSioResult := LibSio_TxString(COPY(stAddr, 3, 6));
    fTerm := TRUE;
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE ((fCancel = FALSE) AND (bSioResult = g_bLibSioOK)) DO BEGIN

    {----------------------------------------------- wait for '=' from TM990 }
    cWait := '=';
    bSioResult := Asm_WaitChars(300, NIL, NIL, pcWait, TRUE);

    {-------- '=' received, store the following four characters (data bytes) }
    IF ((bSioResult <> g_bLibSioCANCEL) AND (cWait = CHR($FF))) THEN BEGIN
      bCnt := 0;
      stAddr := '';

      {---------------------------------------- get four databytes after '=' }
      WHILE ((bCnt < 4) AND (bSioResult = g_bLibSioOK)) DO BEGIN
        bCnt := bCnt + 1;
        bSioResult := LibSio_RxByte(pbRx, iTimeout, TRUE);
        cRx := CHR((ORD(cRx) AND $7F));
        WRITE(cRx);
        stAddr := stAddr + cRx;
      END;

      {----------------------- write data and line terminator to file buffer }
      IF (bSioResult = g_bLibSioOK) THEN BEGIN
        fCancel := (Asm_PutBuffer(stAddr + g_stCRLF) = FALSE);
        fRx := TRUE;
      END;

      {--------------------- compare received address with given end address }
      IF (fCancel = FALSE) THEN BEGIN
        iFromAddr := iFromAddr + 2;
        fDone := (Lib_CmpInt(iFromAddr, iToAddr) = g_bINT_GT);
        fCancel := fDone;
      END;

      {------------------------------------------- request another data word }
      IF (fCancel = FALSE) THEN BEGIN
        bSioResult := LibSio_TxByte(ORD(' '));
      END;
    END;
  END;

  IF (fDone = TRUE) THEN BEGIN
    fCancel := FALSE;
  END;

  {--------------- if some comms were active, get TM990 back to command mode }
  IF (fTerm = TRUE) THEN BEGIN
    bSioResult := LibSio_TxByte(ORD(g_cCR));
  END;

  {--------------------------------------- write exec address to file buffer }
  IF ((fDone = TRUE) AND (iExecAddr <> 0)) THEN BEGIN
    stAddr := g_stCRLF + 'P' + Lib_CnvBinToHex(iExecAddr, 4) + g_stCRLF;
    fCancel := (Asm_PutBuffer(stAddr) = FALSE);
  END;

  {--------------------------------------------------------------- cancelled }
  IF ((fCancel = TRUE) AND (fRx = TRUE)) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. Write buffer to ', g_stFile, ' ? (Y/N) ');
    cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));

    IF (cKbd = 'Y') THEN BEGIN
      fCancel := FALSE;
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF ((fDone = TRUE) AND (fCancel = FALSE)) THEN BEGIN
    File_WriteBuffer;
    bSioResult := g_bLibSioOK;
  END;

  WRITELN;

  Asm_InitSio(FALSE);
  Asm_ReceiveMemFile := bSioResult;
END;
{================================================== eoF Asm_ReceiveMemFile() }

{========================================================= Asm_SendMemFile() }
{ Purpose...: Send .MEM file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_SendMemFile: BYTE;

  VAR bSioResult : BYTE;
  VAR pstLine    : g_tpStLine;
  VAR bLen       : BYTE;
  VAR bNdx       : BYTE;
  VAR cKbd       : CHAR;
  VAR fStart     : BOOLEAN;
  VAR fEmptyLine : BOOLEAN;
  VAR bSemiPos   : BYTE;
  VAR cTx        : CHAR;

BEGIN
  bSioResult := g_bLibSioOK;
  pstLine := PTR(1);
  fStart := TRUE;
  fEmptyLine := FALSE;

  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('Continue (Y/N)? ');
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSioResult := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSioResult = g_bLibSioOK) THEN BEGIN
    Asm_InitSio(TRUE);

    {------------------------------------ make sure TM990 is in command mode }
    bSioResult := Asm_WakeUp;
  END;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSioResult <> g_bLibSioCANCEL)) DO BEGIN
    pstLine := File_GetString(-1, fStart, NIL);
    fStart := FALSE;
    bSemiPos := 0;

    IF (pstLine <> NIL) THEN BEGIN
      bLen := LENGTH(pstLine^);

      {------------------------------------------ check for comments on line }
      IF (bLen <> 0) THEN BEGIN
        bSemiPos := POS(';', pstLine^);

        IF (bSemiPos <> 0) THEN BEGIN
          bLen := bSemiPos - 1;
        END;

        {---------------------------- if comment-only line, print to screen }
        IF ((bLen = 0) AND (bSemiPos = 1)) THEN BEGIN
          WRITELN(pstLine^);
        END;
      END;

      {------------------------------------ write the line to screen and SIO }
      IF (bLen <> 0) THEN BEGIN
        bNdx := 0;

        WHILE (bNdx < bLen) DO BEGIN
          bNdx := (bNdx + 1);
          cTx := pstLine^[bNdx];

          IF ((cTx <> g_cTAB) AND (cTx <> ' ')) THEN BEGIN
            bSioResult := LibSio_TxByte(ORD(cTx));

            {------------- after command character wait for TM990's response }
            IF ((fEmptyLine = TRUE) AND (bNdx = 1)) THEN BEGIN
              bSioResult := Asm_WaitChars(300, NIL, NIL, NIL, TRUE);
            END;
          END;
        END;

        {------------------------- terminate a command with CR instead of SP }
        IF (fEmptyLine = TRUE) THEN BEGIN
          fEmptyLine := FALSE;
          bSioResult := LibSio_TxByte(ORD(g_cCR));

        {------------------------- not a command, get next data word with SP }
        END ELSE BEGIN
          bSioResult := LibSio_TxByte(ORD(' '));
        END;

        bSioResult := Asm_WaitChars(300, NIL, NIL, NIL, TRUE);

      {------------------------------- empty line, either end or new command }
      END ELSE BEGIN
        IF (bSemiPos <> 1) THEN BEGIN
          fEmptyLine := TRUE;
          bSioResult := LibSio_TxByte(ORD(g_cCR));
          bSioResult := Asm_WaitChars(300, NIL, NIL, NIL, TRUE);
        END;
      END;
    END;
  END;

  {------------------------------------------ put TM990 back in command mode }
  IF (fStart = FALSE) THEN BEGIN
    bSioResult := LibSio_TxByte(ORD(g_cCR));
    bSioResult := Asm_WaitChars(300, NIL, NIL, NIL, FALSE);
  END;

  {-------------------------- Asm_WaitChars() always terminates with timeout }
  IF (bSioResult = g_bLibSioTIMEOUT) THEN BEGIN
    bSioResult := g_bLibSioOK;
  END;

  WRITELN;

  {--------------------------------------------------------------- cancelled }
  IF (bSioResult = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  Asm_InitSio(FALSE);
  Asm_SendMemFile := bSioResult;
END;
{===================================================== eoP Asm_SendMemFile() }

{========================================================= Asm_SendAsmFile() }
{ Purpose...: Send .ASM file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Asm_SendAsmFile: BYTE;

  CONST iWAIT_SCREEN = 30;
  CONST iWAIT_LIST   = 80;
  CONST iWAIT_PRINT  = 80;
  CONST iWAIT_CR     = 300;
    
  VAR bSioResult  : BYTE;                        { SIO rx/tx result          }
  VAR iWait       : INTEGER;                     { WaitRx wait time          }
  VAR bLen        : BYTE;                        { string length             }
  VAR bPos        : BYTE;                        { sting position            }
  VAR bNdx        : BYTE;                        { string index              }
  VAR cKbd        : CHAR;                        { char from keyboard        }
  VAR fStart      : BOOLEAN;                     { start flag for file read  }
  VAR fPrinter    : BOOLEAN;                     { printer enabled           }
  VAR cLast       : CHAR;                        { last char from TM990      }
  VAR pcLast      : g_tpChar;                    { pointer to the last char  }
  VAR pstLine     : g_tpstLine;                  { pointer to line from file }
  VAR stTmp       : g_tst;                       { temporary for filtering   }
  VAR stTx        : g_tst;                       { tab-free string to send   }
  VAR stPrinter   : g_tst;                       { line to send to prt/file  }
  VAR pstPrinter  : g_tpst;                      { pointer to print line     }
  VAR stRemainder : g_tst;                       { not yet printed from TM   }       
  VAR fLstFile    : BOOLEAN;                     { list file enabled         }
  VAR stLstFile   : g_tstFile;                   { list file name            }
  VAR oLstFile    : TEXT;                        { list file object          }
     
BEGIN
  bSioResult := g_bLibSioOK;
  fLstFile := FALSE;
  fPrinter := FALSE;
  pcLast := PTR(ADDR(cLast));
  pstPrinter := PTR(ADDR(stPrinter));
  pstLine := PTR(1);
  fStart := TRUE;
  stPrinter := '';
  stRemainder := '';
  iWait := iWAIT_SCREEN;
    
  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN('Assemble ', g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('List to <S>creen | <P>rinter | <F>ile | <ESC> Cancel ');
  cKbd := UPCASE(Lib_GetKey('SsPpFf' + g_cESC, TRUE));
  WRITELN;

  IF (cKbd = g_cESC) THEN BEGIN
    bSioResult := g_bLibSioCANCEL;
  END ELSE BEGIN
    CLRSCR;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
  END;

  {---------------------------------------------------------- open list file }
  IF (cKbd = 'F') THEN BEGIN
    iWait := iWAIT_LIST;
    fLstFile := TRUE;
          
    {--------------------------------------------------- make list file name }
    stLstFile := COPY(g_stFile, 1, POS('.', g_stFile)) + 'LST';
    
    {-------------------------------------------------- create the list file }
    ASSIGN(oLstFile, stLstFile);
    {$I-} REWRITE(oLstFile) {$I+};
    
    IF (IORESULT <> 0) THEN BEGIN
      WRITELN('Error creating ', stLstFile);
      fLstFile := FALSE;
      iWait := iWAIT_SCREEN;
    END;  
  END;
  
  {---------------------------------------- enable printer, reset line count }
  IF (cKbd = 'P') THEN BEGIN
    iWait := iWAIT_PRINT;
    LibPrt_Setup(0, 0, TRUE);
    fPrinter := TRUE;
  END;

  {---------------------------------------------------------------- init SIO }
  IF (bSioResult <> g_bLibSioCANCEL) THEN BEGIN
    Asm_InitSio(TRUE);

    {------------------------------------ make sure TM990 is in command mode }
    bSioResult := Asm_WakeUp;
  END;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSioResult <> g_bLibSioCANCEL)) DO BEGIN
    pstLine := File_GetString(-1, fStart, NIL);
    fStart := FALSE;

    IF (pstLine <> NIL) THEN BEGIN
      bLen := LENGTH(pstLine^);
      bPos := 0;
            
      {---------------------- check for line containing only space(s) or TABs }
      WHILE (bPos < bLen) DO BEGIN
        bPos := bPos + 1;
          
        IF (pstLine^[bPos] > ' ') THEN BEGIN
          bPos := $FF;
        END;
      END;  

      IF (bPos <> $FF) THEN BEGIN
        bLen := 0;
      END;

      {------- if comment-only or empty line, write to printer and list file }
      IF ((bLen = 0) OR (POS(';', pstLine^) = 1)) THEN BEGIN
        bLen := 0;
        stPrinter := pstLine^ + g_stCRLF;
      END ELSE BEGIN
        stPrinter := stRemainder;
      END;

      {---------------------------------------------------- process one line }
      IF (bLen <> 0) THEN BEGIN
        stTx := pstLine^;

        {---------------------------------- remove line comment from tx line }
        bPos := POS(';', stTx);
        
        IF (bPos <> 0) THEN BEGIN
          stTx := COPY(stTx, 1, bPos - 1);
          bLen := LENGTH(stTx);
        END;
          
        {-------------------------------------- replace all TABs with spaces }
        bNdx := 0;
        
        WHILE (bNdx < bLen) DO BEGIN
          bNdx := bNdx + 1;

          IF (stTx[bNdx] = g_cTAB) THEN BEGIN
            stTx[bNdx] := ' ';
          END;
        END;  

        {---------------------- replace double or more spaces with one space }
        stTmp := '';
        cLast := CHR(0);
        bNdx := 0;
        
        WHILE (bNdx < bLen) DO BEGIN
          bNdx := bNdx + 1;

          IF ((cLast <> ' ') OR (stTx[bNdx] <> ' ')) THEN BEGIN
            stTmp := stTmp + stTx[bNdx];
          END;
          
          cLast := stTx[bNdx];
        END;

        {- remove trailing spaces, they cause assembler errors in some cases }        
        WHILE (COPY(stTmp, LENGTH(stTmp), 1) = ' ') DO BEGIN
          stTmp := COPY(stTmp, 1, LENGTH(stTmp) - 1);
        END;
          
        stTx := stTmp;              
        bLen := LENGTH(stTx);
        cLast := CHR(0);
        
        {---------------------------------- send the converted line to TM990 }
        bNdx := 0;

        WHILE ((bNdx < bLen) AND (bSioResult <> g_bLibSioCANCEL)) DO BEGIN
          bNdx := bNdx + 1;
          bSioResult := LibSio_TxByte(ORD(stTx[bNdx]));
          bSioResult := Asm_WaitChars(iWait, NIL, NIL, NIL, TRUE);
        END;

        {--------------------------------------- separate label with a space }
        IF (pstLine^[1] > ' ') THEN BEGIN
          stPrinter := stPrinter + ' ';
        END;
          
        stPrinter := stPrinter + pstLine^;

        {--------------------------------- after a line of source, send a CR }
        IF (bSioResult <> g_bLibSioCANCEL) THEN BEGIN
          bSioResult := LibSio_TxByte(ORD(g_cCR));
          bSioResult := Asm_WaitChars(iWAIT_CR,pstPrinter,pcLast,NIL,TRUE);
        END;

        {--------------------------------- CRs until last rx char is a space }
        WHILE ((cLast <> ' ') AND (bSioResult <> g_bLibSioCANCEL)) DO BEGIN
          bSioResult := LibSio_TxByte(ORD(g_cCR));
          bSioResult := Asm_WaitChars(iWAIT_CR,pstPrinter,pcLast,NIL,TRUE);

          {------------------------------ nothing received, prevent stalling }
          IF ((cLast = CHR(0)) OR (cLast = '?')) THEN BEGIN
            cLast := ' ';
          END;
        END;
      END;

      {------------------------------------------------- printer/file output }
      IF ((fPrinter = TRUE) OR (fLstFile = TRUE)) THEN BEGIN
        bPos := 1;
        
        {----------------------------- print all lines in buffer, one by one }
        WHILE (bPos <> 0) DO BEGIN
          bPos := POS(g_cLF, stPrinter);

          IF (bPos > 0) THEN BEGIN

            IF (fPrinter = TRUE) THEN BEGIN
              fPrinter := LibPrt_Print(CHR(0), COPY(stPrinter, 1, bPos));
            END;
            
            IF (fLstFile = TRUE) THEN BEGIN
              WRITE(oLstFile, COPY(stPrinter, 1, bPos));
            END;              

            stPrinter := COPY(stPrinter, bPos + 1, 255);
          END;
        END;

        {---------------------------- save remainder (unprinted part) of line } 
        IF (LENGTH(stPrinter) <> 0) THEN BEGIN
          stRemainder := stPrinter;
          stPrinter := '';
        END;  
      END;
    END;
  END;

  WRITELN;

  {-------------------------- Asm_WaitChars() always terminates with timeout }
  IF (bSioResult = g_bLibSioTIMEOUT) THEN BEGIN
    bSioResult := g_bLibSioOK;
  END;

  {------------------------------------- let the printer print the last line }
  IF (fPrinter = TRUE) THEN BEGIN
    fPrinter := LibPrt_Print(CHR(0), g_stCRLF);
  END;

  {----------------------------------------------------- close the list file }
  IF (fLstFile = TRUE) THEN BEGIN

    {------------------------------- prevent last sector garbage on printout }
    WRITE(oLstFile, g_cEOF);

    CLOSE(oLstFile);
  END;
    
  {--------------------------------------------------------------- cancelled }
  IF (bSioResult = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled, toggle LOAD switch. ');
  END;

  Asm_InitSio(FALSE);
  Asm_SendAsmFile := bSioResult;
END;
{===================================================== eoF Asm_SendAsmFile() }

{########################################################## EOF TM990ASM.INC }
