{############################################################## TM990BAS.INC }
{ Purpose...: TM990 include file: Send/receive BAS files
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{*************************************************************** definitions }
CONST m_iBAS_BAUD_RATE:  INTEGER  = 300;         { baudrate for .BAS files   }
CONST m_iBAS_TXDLY_CHAR: INTEGER  = 0;           { tx delay between chars    }
CONST m_iBAS_TXDLY_LINE: INTEGER  = 5000;        { tx delay detween lines    }

{************************************************************ implementation }
{============================================================= Bas_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Bas_InitSio(i_fInit: BOOLEAN);

BEGIN

  LibSio_Init(i_fInit, m_iBAS_BAUD_RATE, 'E', 7, 1,
              m_iBAS_TXDLY_CHAR, m_iBAS_TXDLY_LINE);
END;
{========================================================= eoF Bas_InitSio() }

{=============================================================== Bas_TxCmd() }
{ Purpose...: Send a command to target, wait until response arrived
{ Input.....: i_stCmd: Command to send (and wait for)
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Bas_TxCmd(i_stCmd: g_tst): BYTE;

  VAR bSioResult: BYTE;
  VAR bLen:       BYTE;
  VAR bNdx:       BYTE;
  VAR bRx:        BYTE;
  VAR bTx:        BYTE;
  VAR pbRx:       ^BYTE;

BEGIN
  pbRx := PTR(ADDR(bRx));
  bLen := LENGTH(i_stCmd);
  bNdx := 0;
  bSioResult := g_bLibSioOK;

  WHILE ((bNdx < bLen) AND (bSioResult = g_bLibSioOK)) DO BEGIN
    bNdx := bNdx + 1;
    bTx := ORD(i_stCmd[bNdx]);

    IF (LibSio_TxByte(bTx) = g_bLibSioOK) THEN BEGIN
      bSioResult := LibSio_RxByte(pbRx, 2000, FALSE);
      bRx := (bRx AND $7F);

      IF ((bSioResult = g_bLibSioOK) AND (bRx <> bTx)) THEN BEGIN
        bSioResult := g_bLibSioNOK;
      END;
    END;
  END;

  Bas_TxCmd := bSioResult;
END;
{=========================================================== eoF Bas_TxCmd() }

{============================================================== Bas_Wakeup() }
{ Purpose...: Try to connect to TM990 by sending a CR, if alive, TM990
{             sends back a LF $8D sequence
{ Result....: One of g_bLibSio<result>
{=========================================================================== }
FUNCTION Bas_WakeUp: BYTE;

  TYPE tab2 = ARRAY[1..2] OF BYTE;

  VAR bSioResult : BYTE;
  VAR fHint      : BOOLEAN;
  VAR fDone      : BOOLEAN;
  VAR abRx       : tab2;
  VAR pbRx       : ^BYTE;

BEGIN
  fHint := FALSE;
  fDone := FALSE;
  bSioResult := g_bLibSioTIMEOUT;
  pbRx := PTR(ADDR(abRx[1]));

  WRITE('Waking up TM990 ...');

  WHILE (fDone = FALSE) DO BEGIN
    bSioResult := LibSio_TxByte(ORD(g_cCR));

    IF (bSioResult = g_bLibSioOK) THEN BEGIN
      abRx[1] := 0;
      abRx[2] := 0;

      bSioResult := LibSio_RxString(pbRx, NIL, $8D, 1000);

      IF (bSioResult <> g_bLibSioOK) THEN BEGIN

        IF ((bSioResult <> g_bLibSioCANCEL) AND (fHint = FALSE)) THEN BEGIN
          fHint := TRUE;
          WRITE(' (might need to enter UNIT2<CR>) ');
        END ELSE BEGIN
          fDone := (bSioResult = g_bLibSioCANCEL);
        END;
      END

      ELSE BEGIN
        IF ((abRx[1] = $0A) AND (abRx[2] = $8D)) THEN BEGIN
          fDone := TRUE;
          WRITELN(' Ready');
        END;
      END;
    END;
  END;

  Bas_WakeUp := bSioResult;
END;
{========================================================== eoF Bas_WakeUp() }

{============================================================ Bas_SendFile() }
{ Purpose...: Send .BAS file to SIO
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Bas_SendFile: BYTE;

  VAR bSioResult  : BYTE;
  VAR pstLine     : g_tpStLine;
  VAR iBufLine    : INTEGER;
  VAR fLineValid  : BOOLEAN;
  VAR cKbd        : CHAR;
  VAR fStart      : BOOLEAN;

BEGIN
  bSioResult := g_bLibSioOK;

  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_TRANSMIT, g_stFile, ' (', File_GetBufLines, ' lines)');
  WRITE('Continue (Y/N)? ');
  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSioResult := g_bLibSioCANCEL;

  {---------------------------------------------------------------- init SIO }
  END ELSE BEGIN
    CLRSCR;
    Bas_InitSio(TRUE);
    LibSio_RxClear;
    WRITELN(g_stMSG_TRANSMITTING, g_stFile, g_stMSG_CANCEL);
    bSioResult := Bas_WakeUp;
  END;

  pstLine := PTR(1);
  fStart := TRUE;
  iBufLine := 0;

  {----------------------------------------------------------- send the file }
  WHILE ((pstLine <> NIL) AND (bSioResult = g_bLibSioOK)) DO BEGIN
    pstLine := File_GetString(iBufLine, fStart, NIL);
    fStart := FALSE;
    iBufLine := -1;

    {--------------------------------------------- send and display one line }
    IF (pstLine <> NIL) THEN BEGIN

      {------------------------------ do not send lines beginning with a ';' }
      fLineValid := (pstLine^[1] <> ';');
      
      {------------------------------------ write the line to screen and SIO }
      IF (fLineValid = TRUE) THEN BEGIN
        bSioResult := LibSio_TxString(pstLine^ + g_cCR);
      END;

      WRITELN(pstLine^);
    END;
  END;

  WRITELN;

  IF (bSioResult = g_bLibSioCANCEL) THEN BEGIN
    WRITE('Cancelled. ');
  END;

  Bas_InitSio(FALSE);
  Bas_SendFile := bSioResult;
END;
{======================================================== eoP Bas_SendFile() }

{========================================================= Bas_ReceiveFile() }
{ Purpose...: Receive BAS file
{ Result....: One of g_bLibSio<result>
{============================================================================}
FUNCTION Bas_ReceiveFile: BYTE;

  CONST iEND_TIMEOUT : INTEGER = 1500;

  VAR bSioResult   : BYTE;
  VAR cKbd         : CHAR;
  VAR cRx          : CHAR;
  VAR bRx          : BYTE;
  VAR pbRx         : ^BYTE;
  VAR fRx          : BOOLEAN;
  VAR iTimeout     : INTEGER;
  VAR fWriteBuffer : BOOLEAN;

BEGIN
  iTimeout := -1;
  pbRx := PTR(ADDR(cRx));
  fRx := FALSE;
  fWriteBuffer := FALSE;

  {--------------------------------------------------- wait for confirmation }
  CLRSCR;
  WRITELN(g_stMSG_RECEIVE, g_stFile, ' (', File_GetBufSize, ' bytes free)');
  WRITE('Continue (Y/N)? ');

  cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
  WRITELN;

  IF (cKbd = 'N') THEN BEGIN
    bSioResult := g_bLibSioCANCEL;

  {--------------------------------------------- init SIO and wake up TM990 }
  END ELSE BEGIN
    CLRSCR;
    Bas_InitSio(TRUE);
    LibSio_RxClear;
    WRITELN(g_stMSG_RECEIVING, g_stFile, g_stMSG_CANCEL);
    bSioResult := Bas_WakeUp;

    {---------------------------------------------------- send LIST command }
    IF (bSioResult = g_bLibSioOK) THEN BEGIN
      IF (Bas_TxCmd('LIST ' + g_cCR) = g_bLibSioOK) THEN BEGIN
        bSioResult := g_bLibSioNOK;
        LibSio_RxClear;
      END;
    END;
  END;

  {-------------------------------------------- loop until done or cancelled }
  WHILE (bSioResult = g_bLibSioNOK) DO BEGIN
    bSioResult := LibSio_RxByte(pbRx, iTimeout, TRUE);
    cRx := CHR((ORD(cRx) AND $7F));

    {-------------------------------------------- one character was received }
    IF (bSioResult = g_bLibSioOK) THEN BEGIN
      bSioResult := g_bLibSioNOK;
      iTimeout := iEND_TIMEOUT;
      bRx := ORD(cRx);
      fRx := TRUE;

      {-------------------------------------- filter out unwanted characters }
      IF ((bRx<>0) AND (bRx<>6) AND (bRx<>$13) AND (bRx<>$14)) THEN BEGIN

        {------------------------------------ write character to file buffer }
        IF (bRx <> $0A) THEN BEGIN
          IF (File_PutBuffer(cRx) = FALSE) THEN BEGIN
            bSioResult := g_bLibSioCANCEL;
          END;
        END;

        {----------------------------------------- write character to screen }
        IF (bSioResult = g_bLibSioNOK) THEN BEGIN
          WRITE(cRx);
        END;
      END;
    END;
  END;

  {----------------------------------------------------- save buffer to disk }
  IF (bSioResult = g_bLibSioTIMEOUT) THEN BEGIN
    fWriteBuffer := fRx;
    bSioResult := g_bLibSioOK;
  END;

  {--------------------------------------------------------------- cancelled }
  IF ((bSioResult = g_bLibSioCANCEL) AND (fRx = TRUE)) THEN BEGIN
    WRITELN;
    WRITE('Cancelled. Write buffer to ', g_stFile, ' ? (Y/N) ');
    cKbd := UPCASE(Lib_GetKey('YNyn', TRUE));
    fWriteBuffer := (cKbd = 'Y');
  END;

  WRITELN;

  IF (fWriteBuffer = TRUE) THEN BEGIN
    File_WriteBuffer;
    WRITELN;
  END;

  Bas_InitSio(FALSE);
  Bas_ReceiveFile := bSioResult;
END;
{===================================================== eoF Bas_ReceiveFile() }

{########################################################## EOF TM990BAS.INC }
